# -*- coding:Utf-8 -*-
"""
    Gère un décompte écrit et audio de la bombe sous Counter-Strike: Source
"""

import psyco
psyco.full()

import es, os, gamethread, popuplib
from es import server_var
from Decompte import Decompte


info = es.AddonInfo()
info.name = "Complete C4Timer"
info.version = "2.1"
info.author = "Nicolous"
info.url = "http://mattie.info/forums/"
info.description = "Provide a C4 countdown"
info.tags = "c4;countdown"


class CompleteC4CountDown:
    """Classe statique encapsulant le script"""

    __CONFIGURATION_FILE = "addons/eventscripts/complete_c4timer/configuration.txt"

    __decompte = Decompte()

    __popup = None

    class ConVars:
        """Classe interne statique regroupant les ConVars du script"""

        # ConVar publique pour tracer le script sur les moniteurs de serveurs
        c4timer_version = es.ServerVar("c4timer_version",info.version).makepublic()

        c4timer_activation = es.ServerVar("c4timer_activation","0","C4Timer : 1 = activation, 0 = off")
        c4timer_sound_dir = es.ServerVar("c4timer_sound_dir","c4timer","C4Timer : sounds localization from cstrike/sound")
        c4timer_chat_countdown = es.ServerVar("c4timer_chat_countdown","0","C4Timer : 1 = chat countdown, 0 = off")
        c4timer_hud_countdown = es.ServerVar("c4timer_hud_countdown","0","C4Timer : 1 = hud countdown, 0 = off")
        c4timer_center_countdown = es.ServerVar("c4timer_center_countdown","0","C4Timer : 1 = center countdown, 0 = off")
        c4timer_amx_countdown = es.ServerVar("c4timer_amx_countdown","0","C4Timer : 1 = amx countdown, 0 = off")
        c4timer_sound_countdown = es.ServerVar("c4timer_sound_countdown","0","C4Timer : 1 = sound countdown, 0 = off")

        # Récupération d'une référence sur la ConVar mp_c4timer du jeu
        mp_c4timer = es.ServerVar("mp_c4timer")


    @staticmethod
    def __planifieTelechargements():
        """Planifie de façon dynamique les sons à faire télécharger aux joueurs"""

        if int(CompleteC4CountDown.ConVars.c4timer_sound_countdown) and int(CompleteC4CountDown.ConVars.mp_c4timer) >= 0:
            index = int(CompleteC4CountDown.ConVars.mp_c4timer)
            # Bien que la ConVar soit bornée, un es_forcevalue peut forcer une valeur incohérente (par exemple)
            if index < 0:
                index = 0
            # Pour chaque seconde possible du décompte...
            # format : c4timer_<SECONDE>_sec.mp3
            while index >= 0:
                # ... On regarde s'il existe un fichier son correspondant
                nomMod = "sound/%s/c4timer_%s_sec.mp3" % (str(CompleteC4CountDown.ConVars.c4timer_sound_dir),str(index))
                nom = os.getcwd() + "/cstrike/%s" % nomMod
                if os.path.isfile(nom):
                    # Si oui, on l'ajoute aux téléchargements requis pour entrer sur le serveur
                    es.stringtable("downloadables",nomMod)
                index -= 1


    @staticmethod
    def executeConfig():
        """Exécute la configuration du script"""

        # Exécution de la configuration du script
        if os.path.isfile(os.getcwd() + "/cstrike/" + CompleteC4CountDown.__CONFIGURATION_FILE):
            es.ForceServerCommand("es_mexec ../" + CompleteC4CountDown.__CONFIGURATION_FILE)
        else:
            es.dbgmsg(0,"Unable to find " + CompleteC4CountDown.__CONFIGURATION_FILE + " !")

        # Planification automatique des téléchargements
        CompleteC4CountDown.__planifieTelechargements()


    @staticmethod
    def lanceDecompte():
        """Lance le décompte de la bombe"""

        if int(CompleteC4CountDown.ConVars.c4timer_activation):
            CompleteC4CountDown.__decompte.lanceDecompte(int(CompleteC4CountDown.ConVars.mp_c4timer),CompleteC4CountDown.__affiche)


    @staticmethod
    def stoppeDecompte():
        """Stoppe le décompte de la bombe"""
        
        CompleteC4CountDown.__decompte.stoppeDecompte()


    @staticmethod
    def __affiche(tempsRestant):
        """Affiche le décompte"""

        # Si le décompte n'est plus activé on l'arrête
        if not int(CompleteC4CountDown.ConVars.c4timer_activation):
            CompleteC4CountDown.stoppeDecompte()
            return
            
        # Récupération de la liste des ID des joueurs connectés
        playerlist = es.getUseridList()

        # Convertion du temps restant en chaîne de caractères
        tempsRestantStr = str(tempsRestant)

        # Affichage dans le TCHAT
        if int(CompleteC4CountDown.ConVars.c4timer_chat_countdown):
            es.msg("#green",tempsRestantStr)

        # Affichage dans le HUD centré
        if int(CompleteC4CountDown.ConVars.c4timer_hud_countdown):
            es.usermsg("create", "hudcount", "HintText")
            es.usermsg("write", "byte", "hudcount", "1")
            es.usermsg("write", "string", "hudcount",tempsRestantStr)
            for userid in playerlist:
                es.usermsg("send", "hudcount", userid, "0")
            es.usermsg("delete", "hudcount")

        # Affichage centré
        if int(CompleteC4CountDown.ConVars.c4timer_center_countdown):
            es.centermsg(tempsRestantStr)

        # Affichage dans une fenêtre AMX
        if int(CompleteC4CountDown.ConVars.c4timer_amx_countdown):
            CompleteC4CountDown.__popup = popuplib.create("C4CountDown")
            CompleteC4CountDown.__popup.timeout("view",1)
            CompleteC4CountDown.__popup.timeout("send",1)
            CompleteC4CountDown.__popup.addline(tempsRestantStr)
            CompleteC4CountDown.__popup.send(playerlist)
                
        # Lecture du son correspondant
        if int(CompleteC4CountDown.ConVars.c4timer_sound_countdown):
            nomMod = "%s/c4timer_%s_sec.mp3" % (str(CompleteC4CountDown.ConVars.c4timer_sound_dir),tempsRestantStr)
            nomSon = os.getcwd() + "/cstrike/sound/%s" % nomMod
            if os.path.isfile(nomSon):
                commande = "play %s"  % (nomMod)
                es.cexec_all(commande)


def es_map_start(event_var):
    """Fonction automatiquement exécutée par EventScripts à chaque changement de map"""

    CompleteC4CountDown.executeConfig()


def bomb_planted(event_var):
    """Fonction automatiquement exécutée par EventScripts lorsqu'une bombe est amorcée"""

    # Lancement du décompte
    CompleteC4CountDown.lanceDecompte()


def bomb_defused(event_var):
    """Fonction automatiquement exécutée par EventScripts lorsqu'une bombe est désamorcée"""

    # Arrêt du décompte
    CompleteC4CountDown.stoppeDecompte()


def round_end(event_var):
    """Fonction automatiquement exécutée par EventScripts lorsqu'une round se termine"""

    # Arrêt du décompte si l'origine de la fin du round n'est pas l'explosion de la bombe
    if event_var["message"] != "#Target_Bombed":
        CompleteC4CountDown.stoppeDecompte()


def round_start(event_var):
    """Fonction automatiquement exécutée par EventScripts lorsqu'une round démarre"""

    # Arrêt du décompte
    CompleteC4CountDown.stoppeDecompte()
